/* BEGIN: Glossary */
(function (root) {
"use strict";

var glossaries;

function populateGlossaries (id) {
	var items = _Xml.select(head, '//glossary');
	
	glossaries = _.map(_.toArray(_Xml.select(head, '//glossary')), function (glossary) {
		var terms = _.map(_Xml.collect(glossary, 'text'), function (term) {
			return {
				title: _Xml.title(term) || '',
				id:    _Xml.id(term),
				item:  term,
				text:  _Xml.text(term) || ''
			};
		});
		return {
			title: _Xml.title(glossary),
			id:    _Xml.id(glossary),
			terms: terms
		}
	}, []);
	glossaries.push({
		title: 'Глоссарий',
		id:    '__ALL__',
		terms: _.reduce(glossaries, function (memo, glossary) {
			return memo.concat(glossary.terms);
		}, [])
	});
	_.each(glossaries, function (glossary) {
		glossary.terms = _.uniq(glossary.terms, false, function (term) {
			return term.title.toLowerCase() + '__~~__' + term.text.toLowerCase()
		});
	});
}

$(document).delegate('form#glossary-form', 'submit', function (event) {
	event.preventDefault();
	root.executeGlossaryQuery();
});
$(document).delegate('form#glossary-form input#glossary-input', 'keydown keyup keypress click paste input', function (event) {
	filterGlossaryByTimer();
});
$(document).delegate('.eAu-glossary a.eAu-glossary-link', 'click', function (event) {
	event.preventDefault();
	$('form#glossary-form input#glossary-id').val($(this).data('glossaryId'));
	$(this).closest('ul').find('.eAu-glossary-link').removeClass('eAu-glossary-link-active');
	$(this).addClass('eAu-glossary-link-active');
	// clearGlossaryFilterInput();
	// или
	executeGlossaryQuery();
});
$(document).delegate('.eAu-glossary .eAu-glossary-clear', 'click', function (event) {
	event.preventDefault();
	clearGlossaryFilterInput();
});
$(document).delegate('form#glossary-form input[type="image"]', 'click', function (event) {
	event.preventDefault();
	executeGlossaryQuery();
});
$(document).delegate('.eAu-glossary .eAu-glossary-item-term', 'click', function (event) {
	$(this).closest('.eAu-glossary-item').find('.eAu-glossary-item-description').toggle();
	event.preventDefault();
});

root.getGlossary = function (id) {
	if (!glossaries) {
		populateGlossaries(id);
	}
	$window.one('eAu:content:afterchange', function () {
		executeGlossaryQuery();
	});
	text_transform([
		'<div class="eAu-glossary">',
		glossaries.length > 2 ? '<h1 class="eAu"><ul class="eAu-glossary-list"><li><a href="#" class="eAu-glossary-link '+ (!id ? 'eAu-glossary-link-active' : '') +'">Общий глоссарий</a></li>'+ _.reduce(glossaries, function (memo, glossary) {
			if (glossary.id != '__ALL__') {
				memo += '<li><a href="#'+ glossary.id +'" class="eAu-glossary-link '+ (glossary.id == id ? 'eAu-glossary-link-active' : '') +'" data-glossary-id="'+ glossary.id +'">'+ glossary.title +'</a></li>'
			}
			return memo;
		}, '') +'</ul></h1>' : '<h1 class="eAu">Глоссарий</h1>',
		'	<form id="glossary-form" class="eAu-glossary-search-form"><input type="hidden" id="glossary-id" value="'+ id +'"/>',
		'		<table border="0" cellspacing="0" cellpadding="0"><tr>',
		'			<td valign="middle">',
		'				<input value="" type="text" id="glossary-input" class="eAu-input-text eAu-glossary-input" name="glossary-input"/>',
		'			</td>',
		'			<td width="1">',
		'				<input type="image" src="images/buttons/search.gif" title="Найти" class="eAu-glossary-form-submit" />',
		'			</td>',
		'		</tr></table>',
		'	</form>',
		'	<div class="eAu-glossary-items" id="glossary-items-container"></div>',
		'</div>'
	].join('\n'), 'cont');
}

function findGlossary (id) {
	var glossary = _.find(glossaries, function (glossary) {
		return glossary.id == id;
	});
	if (!glossary) {
		glossary = _.find(glossaries, function (glossary) {
			return glossary.id == '__ALL__';
		});
	}
	return glossary;
}

function executeGlossaryQuery () {
	var value = $('form#glossary-form input#glossary-input').val().toLowerCase();
	var glossary = findGlossary($('form#glossary-form input#glossary-id').val());
	var filteredEntries;
	var emptyEntries;

	function filterEmpty(element, index, array) {
		return element.title == '';
	}
	function localeSort(val1, val2) {
		return val1.title.localeCompare(val2.title);
	}
	filteredEntries = [].concat(glossary.terms);
	if (value && value.length == 1) {
		// Starts With
		filteredEntries = _.select(filteredEntries, function (term) {
			return term.title.charAt(0).toLowerCase() == value.charAt(0).toLowerCase();
		});
	} else if (value && value.length > 1) {
		// Contains
		filteredEntries = _.select(filteredEntries, function (term) {
			return term.title.toLowerCase().indexOf(value.toLowerCase()) != -1;
		});
	}
	filteredEntries = filteredEntries.sort(function (a, b) {
		return a.title.localeCompare(b.title);
	});
	emptyEntries = _.select(filteredEntries, function (term) {
		return term.title == '';
	});
	filteredEntries = _.without(filteredEntries, emptyEntries);

	var text;
	var firstChar;
	var fC;

	text = _.reduce(filteredEntries.concat(emptyEntries), function (memo, term) {
		fC = term.title.charAt(0);
		if (firstChar != fC) {
			if (firstChar != null && ( (fC.match(/[A-ZА-Я]/i) && !fC.match(/\d/)) || fC == '' )) {
				memo.push('<div class="eAu-glossary-group-delimiter"/>');
			}
			firstChar = fC;
		}
		var title = term.title ? term.title : '<i>не указано название термина</i>';
		var div = '<div id="eAu-glossary-item-'+ term.id +'" class="eAu-glossary-item-description" style="display: '+ (term.visible ? 'block' : 'none') +';">'+ term.text +'</div>';
		var link = '<a href="index.htm" data-glossary-term-id="'+ term.id +'" data-glossary-id="'+ glossary.id +'" target="_top" class="eAu-glossary-item-term">' + title + '</a>';

		memo.push('<div class="eAu-glossary-item">'+ link + div +'</div>');
		return memo;
	}, []);
	if (filteredEntries.length + emptyEntries.length == 0) {
		if (value.length == 1) {
			text.push('<div class="eAu-glossary-item"><div class="eAu-nothing-found">Слова, начинающиеся на букву «<b>'+ value +'</b>», отсутствуют в глоссарии<br />Попробуйте ввести больше символов, что-бы произвести поиск по подстроке, или <a href="index.htm" target="_top" class="eAu-glossary-clear">произведите очистку поля поиска</a></div></div>');
		} else {
			text.push('<div class="eAu-glossary-item"><div class="eAu-nothing-found">Слова, содержащие подстроку «<b>'+ value +'</b>», отсутствуют в глоссарии<br />Попробуйте <a href="index.htm" target="_top" class="eAu-glossary-clear">произвести очистку поля поиска</a></div></div>');
		}
	}
	$('#glossary-items-container').html('<div class="eAu-glossary-items">' + text.join('') + '</div>');
}
var filterGlossaryByTimer = _.debounce(executeGlossaryQuery, 200);

function clearGlossaryFilterInput () {
	$('form#glossary-form input#glossary-input').val('');
	executeGlossaryQuery();
	try {
		$('#glossary-input').focus();
	} catch (error) {}
}


})(window);
/* END: Glossary */
