(function() {

var root = this;

if (_.isUndefined(root.eAu)) { root.eAu = {}; }
var _eau = root.eAu;

_eau.emptyFunction = function () { };
_eau.K = function (x) { return x; };

/**
 * Довольно забавный способ получения списка аргументов функции.
 *
 * Взято из Prototype
 */
_eau.argumentNames = function(f) {
	if (!_.isFunction(f)) { return []; }
	var names = f.toString().match(/^[\s\(]*function[^(]*\(([^\)]*)\)/)[1]
		.replace(/\s+/g, '').split(',');
	return names.length == 1 && !names[0] ? [] : names;
};
function toQueryPair(key, value) {
	if (_.isUndefined(value)) return key;
	return key + '=' + encodeURIComponent(value == null ? '' : String(value));
}
_eau.toQueryString = function (hash) {
	return _.reduce(hash, function(results, values, key) {
		key = encodeURIComponent(key);

		if (values && typeof values == 'object') {
			if (_.isArray(values))
				return results.concat(_.map(values, _.bind(toQueryPair, null, key)));
		} else results.push(toQueryPair(key, values));
		return results;
	}, []).join('&');
};
_eau.toJSON = function(/* ... */) {
}

/**
 * Метод wrap, аналогичный тому, что был в Prototype.js
 * отличается от _.wrap различными *this*
 */
_eau.wrap = function(func, wrapper) {
	return function() {
		var args = [_.bind(func, this)].concat(_.toArray(arguments));
		return wrapper.apply(this, args);
	};
};

/**
 * Адаптированный вариант OOP из Prototype.js
 * на основе Underscore
 *
 * Based on Alex Arnell's inheritance implementation.
 */
_eau.Class = {
	create: function() {
		var parent = null, properties = arguments;
		if (_.isFunction(properties[0])) {
			parent     = _.first(properties);
			properties = _.rest(properties);
		}

		function klass() {
			this.initialize.apply(this, arguments);
		}

		_.extend(klass, _eau.Class.Methods);
		klass.superclass = parent;
		klass.subclasses = [];

		if (parent) {
			var subclass = function() { };
			subclass.prototype = parent.prototype;
			klass.prototype = new subclass;
			parent.subclasses.push(klass);
		}

		for (var i = 0; i < properties.length; i++)
			klass.addMethods(properties[i]);

		if (!klass.prototype.initialize)
			klass.prototype.initialize = _eau.emptyFunction;

		klass.prototype.constructor = klass;

		return klass;
	}
};
_eau.Class.Methods = {
	addMethods: function(source) {
		var ancestor   = this.superclass && this.superclass.prototype;
		var properties = _.keys(source);

		if (!_.keys({ toString: true }).length)
			properties.push("toString", "valueOf");

		for (var i = 0, length = properties.length; i < length; i++) {
			var property = properties[i], value = source[property];
			if (ancestor && _.isFunction(value) &&
					_.first(_eau.argumentNames(value)) == "$super") {
				var method = value;
				value = _eau.wrap((function(m) {
					return function() { return ancestor[m].apply(this, arguments) };
				})(property), method);

				value.valueOf = _.bind(method.valueOf, method);
				value.toString = _.bind(method.toString, method);
			}
			this.prototype[property] = value;
		}

		return this;
	}
};

/**
 * Строковые функции
 */
if (_.isUndefined(_eau.String)) { _eau.String = {}; }
var _string = _eau.String;

_string.trim = function(str) {
	if (!str) { return str }
	str = str.replace(/^(\s|\u00A0)+/, '');
	for (var i = str.length - 1; i >= 0; i--) {
		if (/[^\u00A0\s]/.test(str.charAt(i))) {
			str = str.substring(0, i + 1);
			break;
		}
	}
	return str;
}
_string.stripTags = function(str) {
	if (!str) { return str }
	return str.replace(/<\/?[^>]+>/gi, '');
}
_string.escape = function(str) {
	if (!str) { return str }
	return str.replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/&/g, '&amp;');
}
_string.normalizeSpaces = function(str, trim) {
	if (!str) { return str }
	trim = trim ? _string.trim : _eau.K;
	return trim(str.replace(/\s+/g, ' '));
}
_string.$w = function() {
}
_string.toQueryParams = function(str) {
	if (!str) { return str; }
	var match = _string.trim(str).match(/([^?#]*)(#.*)?$/);
	if (!match) return { };

	return _.reduce(match[1].split('&'), function(hash, pair) {
		if ((pair = pair.split('='))[0]) {
			var key = decodeURIComponent(pair.shift());
			var value = pair.length > 1 ? pair.join('=') : pair[0];
			if (value != root.undefined) value = decodeURIComponent(value);

			if (key in hash) {
				if (!_.isArray(hash[key])) hash[key] = [hash[key]];
				hash[key].push(value);
			}
			else hash[key] = value;
		}
		return hash;
	}, { });
}
_string.RegExpEscape = function(str) {
	return String(str).replace(/([.*+?^=!:${}()|[\]\/\\])/g, '\\$1');
}

/**
 * Функции массива
 */
if (_.isUndefined(_eau.Array)) { _eau.Array = {}; }
var _array = _eau.Array;

_array.shuffle = function(arr, deep, clone) {
	if (!_.isArray(arr)) { return arr; }
	
	if (clone) arr = _.clone(arr);
	
	var i = arr.length, j, t;
	while( i ) {
		j = Math.floor( ( i-- ) * Math.random() );
		t = deep ? _array.shuffle(arr[i], deep, clone) : arr[i];
		arr[i] = arr[j];
		arr[j] = t;
	}
	return arr;
}

if (_.isUndefined(_eau.Node)) { _eau.Node = {}; }
var _node = _eau.Node;

// DOM level 2 ECMAScript Language Binding
_.extend(_node, {
	ELEMENT_NODE:                1,
	ATTRIBUTE_NODE:              2,
	TEXT_NODE:                   3,
	CDATA_SECTION_NODE:          4,
	ENTITY_REFERENCE_NODE:       5,
	ENTITY_NODE:                 6,
	PROCESSING_INSTRUCTION_NODE: 7,
	COMMENT_NODE:                8,
	DOCUMENT_NODE:               9,
	DOCUMENT_TYPE_NODE:          10,
	DOCUMENT_FRAGMENT_NODE:      11,
	NOTATION_NODE:               12
});

if (_.isUndefined(_eau.Event)) { _eau.Event = {}; }
var _event = _eau.Event;

_.extend(_event, {
	KEY_BACKSPACE: 8,
	KEY_TAB:       9,
	KEY_RETURN:   13,
	KEY_ESC:      27,
	KEY_LEFT:     37,
	KEY_UP:       38,
	KEY_RIGHT:    39,
	KEY_DOWN:     40,
	KEY_DELETE:   46,
	KEY_HOME:     36,
	KEY_END:      35,
	KEY_PAGEUP:   33,
	KEY_PAGEDOWN: 34,
	KEY_INSERT:   45
});

if (_.isUndefined(_eau.UOptions)) { _eau.UOptions = {}; }
if (_.isUndefined(_eau.UOptions.Button)) { _eau.UOptions.Button = {}; }

})();