(function() {

if (!self.eAu) { self.eAu = {}; }
if (eAu.DS) { return; }

var dataProviders = {},
    providerNames = [],
    providerMethods = { required: ['init', 'flush', 'set', 'canSet'] };
var storedData = {};

function dataEach(d, callback) {
	for (var key in d) {
		if (!_.isString(d[key])) { continue; }
		callback.call(null, key, d[key]);
	}
}

function eachProviderInvokeMethod(methodName, params, filter) {
	params = params || [];
	filter = _.isFunction(filter)
		? filter
		: function() { return true; }

	var returnValues = [];
	for (var i = providerNames.length - 1; i >= 0; --i) {
		var name     = providerNames[i];
		var provider = dataProviders[name];
		var method   = provider[methodName];

		if (!_.isFunction(method) || !filter(provider)) {
			continue;
		}

		var complexParams = [];
		for (var j = 0; j < params.length; ++j) {
			if (_.isFunction(params[j])) {
				complexParams.push(
					params[j](provider)
				);
			} else {
				complexParams.push(params[j]);
			}
		}

		var retVal = {
			value: method.apply(provider, complexParams)
		};

		retVal.provider = name;
		returnValues.push(retVal);
	}
	return returnValues;
}

function updateLastAccessTime() {
	// Время последнего доступа к курсу
	storedData.t = parseInt((new Date()).getTime() / 1000 / 60, 10);
}

function registerDataProvider(providerName, provider) {
	if (!_.isUndefined(dataProviders[providerName])) {
		// Такой провайдер уже зарегистрирован
		return false;
	}

	var validDataProvider = true;
	for (var i = 0, methodName; methodName = providerMethods.required[i++];) {
		if (!_.isFunction(provider[methodName])) {
			validDataProvider = false;
			break;
		}
	}

	var isAvailable = _.isFunction(provider.isAvailable)
		? provider.isAvailable()
		: provider.isAvailable;

	if (!validDataProvider || !isAvailable) {
		return false;
	}

	dataProviders[providerName] = provider;

	providerNames.push(providerName);

	providerNames = providerNames.sort(function(name1, name2) {
		return (dataProviders[name2].priority || 1) - (dataProviders[name1].priority || 1);
	});
}

function initializeData(id, initData) {
	var retvalues = eachProviderInvokeMethod('init', [id]);

	for (var i = retvalues.length - 1; i >= 0; --i) {
		storedData = _.extend(
			storedData,
			retvalues[i].value
		);
	}
	_.extend(storedData, initData || {});
	updateLastAccessTime();
}

function setVariable(name, value) {
	if (!_.isString(value)) {
		throw "eAu.DS class can store only strings";
		return false;
	}
	
	storedData[name] = value;

	if (!canSetVariable(name, value)) {
		return false;
	}

	eachProviderInvokeMethod('set', [function(provider) {
		return filterUnsupportedVariablesForProvider(provider);
	}]);

	if (navigator.userAgent.indexOf('Opera') > -1) {
		flushData();
	}
	return true;
}

function filterUnsupportedVariablesForProvider(provider, data) {
	data = data || storedData;
	var tempData = _.clone(data);
	dataEach(data, function(key, value) {
		if (key.charAt(0) == '!' && !provider.isPermament) {
			delete tempData[key];
		} else if (key.charAt(0) == '|' && !provider.isExtended) {
			delete tempData[key];
		}
	});
	return tempData;
}

function canSetVariable(name, value) {
	var tempData = _.clone(storedData);
	tempData[name] = value;

	var isPermamentVariable = name.charAt(0) == '!';
	var isExtendedVariable  = name.charAt(0) == '|';

	// Запрос пошлём только тем провайдерам,
	// которые могут хранить данный тип переменных
	var retvalues = eachProviderInvokeMethod(
		'canSet',
		[function(provider) {
			return filterUnsupportedVariablesForProvider(provider, tempData);
		}],
		function(provider) {
			if (isPermamentVariable && !provider.isPermament) {
				return false;
			} else if (isExtendedVariable && !provider.isExtended) {
				return false;
			}
			return true;
		}
	);

	var canSet = false;
	for (var i = retvalues.length - 1; i >= 0; --i) {
		canSet = canSet || retvalues[i].value;
	}

	return canSet;
}

function unsetVariable(name) {
	delete storedData[name];

	eachProviderInvokeMethod('set', [function(provider) {
		return filterUnsupportedVariablesForProvider(provider);
	}]);

	if (navigator.userAgent.indexOf('Opera') > -1) {
		flushData();
	}
}

function getVariable(name) {
	return storedData[name];
}

function hasVariable(name) {
	return !_.isUndefined(storedData[name]);
}

function flushData() {
	updateLastAccessTime();
	eachProviderInvokeMethod('set', [function(provider) {
		return filterUnsupportedVariablesForProvider(provider);
	}]);
	eachProviderInvokeMethod('flush', []);
}

function clearVariables(type) {
	var validTypes = 'all permament extended temp'.split(' ')

	type = (type || 'temp')
		.toLowerCase()
		.split(/\s+/);

	var types = [];
	for (var i = 0; i < type.length; ++i) {
		if (_.indexOf(validTypes, type[i]) == -1) {
			continue;
		}
		if (type[i] == 'all') {
			types = ['all'];
			break;
		}
		types.push(type[i]);
	}
	for (var i = 0; i < types.length; ++i) {
		dataEach(storedData, function(key, value) {
			if (types[i] == 'temp' && (key.charAt(0) == '!' || key.charAt(0) == '|')) {
				return;
			}
			if (types[i] == 'permament' && key.charAt(0) != '!') {
				return;
			}
			if (types[i] == 'extended' && key.charAt(0) != '|') {
				return;
			}

			delete storedData[key];
		});
	}
	flushData();
}

registerDataProvider('cookie', (function() {
	var Enc = encodeURIComponent;
	var Dec = function () {
		try {
			return decodeURIComponent.apply(null, arguments);
		} catch (error) {
			return null;
		}
	}
	var Prefix = 'eAu_';
	var Expire = 31;
	var Data = {};
	var Cid;

	function set(name, value, daysToExpire) {
		var expire = '';
		if (!_.isUndefined(daysToExpire)) {
			var d = new Date();
			d.setTime(d.getTime() + (86400000 * parseFloat(daysToExpire)));
			expire = '; expires=' + d.toGMTString();
		}
		return (document.cookie = Enc(Prefix + name) + '=' + Enc(value || '') + expire);
	}

	function get(name) {
		var cookie = String(document.cookie).match(new RegExp('(^|;)\\s*' + Enc(Prefix + name) + '=([^;\\s]*)'));
		return (cookie ? Dec(cookie[2]) : null);
	}

	function erase(name) {
		set(name, '', -1);
	}

	function getAllCookies(withPrefix) {
		if (!eAu.String.trim(String(document.cookie || ''))) {
			return {};
		}

		var cooks = {};

		var cookies = String(document.cookie).split(/;\s*/);
		for (var i = cookies.length - 1; i >= 0; --i) {
			var cook = cookies[i].split('=');
			var key = Dec(cook[0]);
			if (!key || (withPrefix && key.indexOf(Prefix) != 0)) {
				continue;
			}
			if (withPrefix) {
				key = key.replace(new RegExp('^'+Prefix), '');
			}
			cooks[key] = Dec(cook[1]);
		}
		return cooks;
	}
	
	function checkSpace(dataObject) {
		var AllCookies = getAllCookies();
		var cookieLength = 0;
		var eAuCookieLength = 0;
		var cookiesCount = 1;
		var eAuCookiesCount = 1;

		// Удаляем из списка куку текущего курса,
		// т.к. сохранённая версия явно устарела
		delete AllCookies[Prefix + Cid];

		// И подсчитываем длину сохранённых кук
		dataEach(AllCookies, function(key, value) {
			if (key.indexOf(Prefix) == 0) {
				eAuCookiesCount++;
				eAuCookieLength += Enc(key).length + Enc(value).length + 1;
			}
			cookiesCount++;
			cookieLength += Enc(key).length + Enc(value).length + 1;
		});
		
		// Добавляем длину куки для наших данных
		var ourCookieLength = Enc(Prefix + Cid).length + 1 + Enc(JSON.stringify(dataObject)).length - 6; // 6 == encodeURIComponent('{}').length

		eAuCookieLength += (ourCookieLength + (eAuCookiesCount - 1) * 2);
		cookieLength    += (ourCookieLength + (cookiesCount - 1) * 2);

		return !!(eAuCookieLength <= 3000 && cookieLength <= 4000);
	}

	function eraseOldPrefixedCookies() {
		var PrefixedCookies = getAllCookies(true);
		var Cids = [];

		delete PrefixedCookies[Cid];

		dataEach(PrefixedCookies, function(key, value) {
			Cids.push(key);
			try {
				PrefixedCookies[key] = JSON.parse('{' + PrefixedCookies[key] + '}');
			} catch (error) {}
		});

		Cids = Cids.sort(function(Cid1, Cid2) {
			return PrefixedCookies[Cid1].t - PrefixedCookies[Cid2].t;
		});
		
		if (Cids.length >= 1) {
			erase(Cids.pop());
			return true;
		}
		return false;
	}
	
	function CheckSpaceAndEraseOld(dataObject) {
		if (!checkSpace(dataObject)) {
			eraseOldPrefixedCookies();
		}

		return checkSpace(dataObject);
	}

	return {
		init: function(cid) {
			Cid = cid;
			// Формат хранения кук - {prefix}{cid}
			// Находим все куки у которых есть префикс
			var allPrefixedCookies = getAllCookies(true);

			if (allPrefixedCookies[Cid]) {
				try {
					Data = JSON.parse('{' + allPrefixedCookies[Cid] + '}');
				} catch (error) {}
			}

			return Data || {};
		},
		// Реально записывает данные в формате, пригодном для dataProvider
		flush: function() {
			CheckSpaceAndEraseOld(Data);

			var jsonData = JSON.stringify(Data);
			set(Cid, jsonData.substr(1, jsonData.length - 2), Expire);

			return true;
		},
		// проверяет хватит-ли места для устновки нового значения
		canSet: function(dataObject) {
			return checkSpace(dataObject);
		},
		// Устанавливает значение
		set: function(dataObject) {
			Data = _.extend({}, dataObject);
		},
		isAvailable: function() {
			if (eAu.LMS.ok()) {
				return false;
			}
			if (navigator.cookieEnabled) {
				return !!navigator.cookieEnabled;
			} else {
				set('_test', '1');
				var retVal = get('_test') === '1';
				erase('_test');
				return !!retVal;
			}
		},
		isPermament: true
	};
})());

registerDataProvider('database', (function() {
	var DatabaseInstance = null;
	var CourseData = {};
	var HasCidRow = false;
	var Cid;
	var CidId;
	var IsGoogleGears = false;
	var ret;

	function createSqlTables() {
		var CreateFirstTableSql = [
			'CREATE TABLE IF NOT EXISTS eAu_CourseIds (',
			'	id       INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,',
			'	CourseId VARCHAR(128) NOT NULL UNIQUE',
			');'
		].join('\n');
		var CreateSecondTableSql = [
			'CREATE TABLE IF NOT EXISTS eAu_Variables (',
			'	id           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,',
			'	CidId        INTEGER NOT NULL,',
			'	VariableName VARCHAR(512) NOT NULL,',
			'	Value        TEXT',
			');'
		].join('\n');
		var CreateIndexSql = [
			'CREATE UNIQUE INDEX IF NOT EXISTS Cid_VariableName_unique',
			'ON eAu_Variables (CidId, VariableName);'
		].join('\n');

		if (IsGoogleGears) {
			var rows = DatabaseInstance.execute(CreateFirstTableSql);
			rows.close();
			rows = DatabaseInstance.execute(CreateSecondTableSql);
			rows.close();
			rows = DatabaseInstance.execute(CreateIndexSql);
			rows.close();
		} else {
			DatabaseInstance.transaction(function (transaction) {
				transaction.executeSql(CreateFirstTableSql, [], function() {});
				transaction.executeSql(CreateSecondTableSql, [], function() {});
				transaction.executeSql(CreateIndexSql, [], function() {});
			});
		}
	}

	function getCidId() {
		var id = false;
		var SelectCidIndexSql = 'SELECT id FROM eAu_CourseIds WHERE CourseId=?';
		if (IsGoogleGears) {
			var rows = DatabaseInstance.execute(SelectCidIndexSql, [Cid]);
			if (rows && rows.isValidRow()) {
				id = parseInt(rows.fieldByName('id'), 10);
				rows.close();
			}
		} else {
			DatabaseInstance.transaction(function (transaction) {
				transaction.executeSql(SelectCidIndexSql, [Cid],
					function(transaction, data) {
						if (data.rows.length < 1) { return; }
						id = parseInt(data.rows.item(0).id, 10);
					}
				);
			});
		}
		return id;
	}

	function createCidId() {
		var CreateCidIndexSql = 'INSERT INTO eAu_CourseIds (CourseId) VALUES (?)';

		if (IsGoogleGears) {
			var rows = DatabaseInstance.execute(CreateCidIndexSql, [Cid]);
			if (rows) {
				rows.close();
			}
		} else {
			DatabaseInstance.transaction(function (transaction) {
				transaction.executeSql(CreateCidIndexSql, [Cid], function() { });
			});
		}

		return getCidId();
	}

	function getStoredData() {
		var GetDataSql = 'SELECT VariableName, Value FROM eAu_Variables WHERE CidId=?';

		if (IsGoogleGears) {
			var rows = DatabaseInstance.execute(GetDataSql, [CidId]);
			if (!rows || !rows.isValidRow()) {
				return;
			}
			while(rows.isValidRow()) {
				CourseData[rows.fieldByName('VariableName')] = rows.fieldByName('Value');
				rows.next();
			}
			rows.close();
		} else {
			DatabaseInstance.transaction(function (transaction) {
				transaction.executeSql(GetDataSql, [CidId],
					function(transaction, data) {
						for (var i = 0; i < data.rows.length; ++i) {
							CourseData[data.rows.item(i).VariableName] = data.rows.item(i).Value;
						}
					}
				);
			});
		}
	}
	
	function diff(Old, New) {
		var removed = [],
		    added = [];

		for (var key in Old) {
			if (!_.isString(Old[key]) || !_.isUndefined(New[key])) { continue; }

			removed.push(key);
		}
		for (var key in New) {
			if (!_.isString(New[key]) || !_.isUndefined(Old[key])) { continue; }

			added.push(key);
		}

		return {
			added: added,
			removed: removed,
			length: added.length + removed.length
		};
	}
	
	var OldData = {};

	ret = {
		init: function(cid) {
			if (!DatabaseInstance) {
				return CourseData;
			}
			Cid = cid;

			createSqlTables();

			CidId = getCidId();
			if (CidId === false) {
				CidId = createCidId();
			}

			getStoredData();

			OldData = CourseData;
			return CourseData;
		},
		// Реально записывает данные в формате, пригодном для dataProvider
		flush: function() {
			var iterator;
			var SqlQuery = 'INSERT OR REPLACE INTO eAu_Variables (CidId, VariableName, Value) VALUES (?, ?, ?);';
			var RemoveSqlQuery = 'DELETE FROM eAu_Variables WHERE CidId=? AND VariableName=?;';

			var Diff = diff(OldData, CourseData);
			if (!Diff.length) {
				return;
			}

			iterator = IsGoogleGears
				? function(data) {
				  	try {
				  		var rows = DatabaseInstance.execute(RemoveSqlQuery, data);
				  		rows.close();
				  	} catch(error) { }
				  }
				: function (data) {
				  	DatabaseInstance.transaction(function (transaction) {
				  		transaction.executeSql(RemoveSqlQuery, data);
				  	});
				  };
			for (var i = 0; i < Diff.removed.length; ++i) {
				iterator([CidId, Diff.removed[i]]);
			}

			iterator = IsGoogleGears
				? function(key, value) {
				  	var data = [CidId, key, value];
				  	try {
				  		var rows = DatabaseInstance.execute(SqlQuery, data);
				  		rows.close();
				  	} catch(error) { }
				  }
				: function(key, value) {
				  	var data = [CidId, key, value];
				  	DatabaseInstance.transaction(function (transaction) {
				  		transaction.executeSql(SqlQuery, data);
				  	});
				  };
			dataEach(CourseData, iterator);
			OldData = CourseData;
		},
		// проверяет хватит-ли места для устновки нового значения
		canSet: function(dataObject) {
			return true;
		},
		// Устанавливает значение
		set: function(dataObject) {
			CourseData = _.extend(dataObject, {});
		},
		isAvailable: function() {
			var w = window;
			var g = w.google;
			if (!w.openDatabase && !(g && g.gears && g.gears.factory)) {
				return false;
			}
			IsGoogleGears = !w.openDatabase;

			if (!IsGoogleGears) {
				var DisplayName = 'База данных для хранения информации о состоянии изучения курсов, созданных в eAuthor';
				try {
					DatabaseInstance = window.openDatabase('eAu_CourseDataStorage', '1.0', DisplayName);
				} catch (error) {
					return false;
				}
			} else {
				g.gears.factory.getPermission('Учебный Курс');
				if (!g.gears.factory.hasPermission) {
					return false;
				}
				try {
					DatabaseInstance = g.gears.factory.create('beta.database');
					DatabaseInstance.open('eAu_CourseDataStorage');
				} catch (error) {
					return false;
				}
			}
			return !!DatabaseInstance;
		},
		isPermament: true,
		isExtended: true,
		priority: 100
	};

	var interval = setInterval(function() {
		ret.flush();
	}, 100);
	$(window).bind('unload', function() {
		clearInterval(interval);
	});

	return ret;
})());

registerDataProvider('lms', (function() {
	var CourseData = {};
	var Cid;
	var ret;
	var isAvailable = false;
	var unsafeRegExp = new RegExp("[!'()*\\-._~]", "g");

	function safeEncode (str) {
		unsafeRegExp.lastIndex = 0;
		return encodeURIComponent(str).replace(unsafeRegExp, function (match) {
			return '%' + match.charCodeAt(0).toString(16).toUpperCase();
		});
	}

	function getStoredData() {
		var LMSSuspendData = eAu.LMS.getSuspendData(),
		    data;

		if (_.isString(LMSSuspendData) && LMSSuspendData != '')  {
			if (LMSSuspendData.charAt(0) == '@') {
				try {
					LMSSuspendData = decodeURIComponent(LMSSuspendData.substr(1));
				} catch (error) {}
			}
			try { data = JSON.parse(LMSSuspendData) } catch (error) {}
		}
		(!data || data.__cid != Cid) && (data = {});

		return data;
	}

	ret = {
		init: function(cid) {
			Cid = cid;
			CourseData = getStoredData();
			return CourseData;
		},
		// Реально записывает данные в формате, пригодном для dataProvider
		flush: function() {
			eAu.LMS.setSuspendData(
				'@' + safeEncode(JSON.stringify(CourseData))
			);
		},
		// проверяет хватит-ли места для устновки нового значения
		canSet: function(dataObject) {
			return true;
		},
		// Устанавливает значение
		set: function(dataObject) {
			CourseData = _.extend({}, dataObject);
			CourseData.__cid = Cid;
		},
		isAvailable: function () {
			return eAu.LMS.ok();
		},
		isPermament: true,
		isExtended:  true,
		priority:    300
	};

	var interval = setInterval(function() {
		ret.flush();
	}, 20000);
	$(window).bind('unload', function() {
		clearInterval(interval);
	});

	return ret;
})());

$(window).bind('unload', flushData);

eAu.DS = {
	init:  initializeData,
	get:   getVariable,
	set:   setVariable,
	can:   canSetVariable,
	unset: unsetVariable,
	has:   hasVariable,
	flush: flushData,
	clear: clearVariables
};

})();
