<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="html" encoding="UTF-8" />

	<xsl:param name="element-title"/>
	<xsl:param name="parent-id"/>
	<xsl:param name="transformed-element-id"/>

	<xsl:param name="appendix-id"/>
	<xsl:param name="main-id"/>

	<!-- Удаляет фигурные скобки вначале и вконце строки -->
	<xsl:template name="stripBraces">
		<xsl:param name="string"/>
		<xsl:choose>
			<xsl:when test="substring($string,1,1)='{' and substring($string,string-length($string))='}'">
				<xsl:value-of select="translate(substring($string,2,string-length($string)-2), '-', '_')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$string"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- Генерирует более приемлемый id на основе DB_ID -->
	<xsl:template name="genID">
		<xsl:param name="type">ID</xsl:param>
		<xsl:param name="id" />
		<xsl:value-of select="$type" />_<xsl:call-template name="stripBraces">
			<xsl:with-param name="string" select="$id" />
		</xsl:call-template><xsl:text>_EU</xsl:text>
	</xsl:template>

	<!-- Нормализация значения атрибута @width у тега <object> -->
	<xsl:template name="getWidth">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@fs = 'yes'">
				<xsl:text>auto</xsl:text>
			</xsl:when>
			<xsl:when test="$node/@width and $node/@width != '0' and $node/@width != ''">
				<xsl:value-of select="$node/@width"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$node/@type = 'image'">
						<xsl:text>auto</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>800</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- Нормализация значения атрибута @height у тега <object> -->
	<xsl:template name="getHeight">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@fs = 'yes'">
				<xsl:text>auto</xsl:text>
			</xsl:when>
			<xsl:when test="$node/@height and $node/@height != '0' and $node/@height != ''">
				<xsl:value-of select="$node/@height"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$node/@type = 'image'">
						<xsl:text>auto</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>600</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="getAlign">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@align = 'center' or $node/@align = 'left' or $node/@align = 'right'">
				<xsl:value-of select="$node/@align"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>left</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Универсальный шаблон для отображения объектов -->
	<xsl:template match="object">
		<xsl:param name="type"/>
		<xsl:param name="hide-nodes"/>
		<xsl:if test="
				    not(@visibility = 'hidden')
				and (not(@visible = 'false') or not(@default = 'false'))
			">
			<div class="eAu-object">
				<xsl:attribute name="id">
					<xsl:call-template name="genID">
						<xsl:with-param name="type">eAu-node-id</xsl:with-param>
						<xsl:with-param name="id"><xsl:value-of select="@DB_ID"/></xsl:with-param>
					</xsl:call-template>
				</xsl:attribute>
				<xsl:if test="$hide-nodes = 'yes'">
					<xsl:attribute name="style">display: none;</xsl:attribute>
				</xsl:if>
				<xsl:if test="$element-title and not(@visibility = 'hidden') and not(../@show-title = 'no')">
					<h1 class="eAu"><xsl:value-of select="$element-title"/></h1>
				</xsl:if>
				<xsl:if test="@title and normalize-space(@title) != '' and @show-title = 'yes'">
					<h2 class="eAu"><xsl:value-of select="@title"/></h2>
				</xsl:if>
				<xsl:if test="not(@visible = 'false')">
					<div class="eAu-object-link">
						<img alt="{@type}" vspace="2" hspace="2" align="absmiddle" border="0" src="images/media/{@type}.gif"/>
						<a href="index.htm" InnerLink="EUL:{@DB_ID}" target="_top"><xsl:value-of select="@title" /></a>
					</div>
				</xsl:if>
				<xsl:if test="@default = 'true'">
					<div class="eAu-object-embed">
						<xsl:variable name="width"><xsl:call-template name="getWidth"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="height"><xsl:call-template name="getHeight"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="align"><xsl:call-template name="getAlign"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="id"><xsl:call-template name="genID"><xsl:with-param name="id" select="@DB_ID" /></xsl:call-template></xsl:variable>
						<xsl:choose>
							<xsl:when test="@type = 'image' and ($width = 'auto' or $height = 'auto') and not(@fs = 'yes')">
								<div class="eAu-object-alignment-container eAu-object-alignment-{$align}">
									<img alt="{@title}" title="{@title}" src="{@src}">
										<xsl:if test="$width != 'auto'">
											<xsl:attribute name="width"><xsl:value-of select="$width"/></xsl:attribute>
										</xsl:if>
										<xsl:if test="$height != 'auto'">
											<xsl:attribute name="height"><xsl:value-of select="$height"/></xsl:attribute>
										</xsl:if>
									</img>
									<br clear="all" />
								</div>
							</xsl:when>
							<xsl:otherwise>
								<div class="eAu-object-alignment-container eAu-object-alignment-{$align}">
									<div style="" class="eAu-object-alignment">
										<xsl:attribute name="style">
											<xsl:if test="$width != 'auto'">width: <xsl:value-of select="$width"/>px;</xsl:if>
											<xsl:if test="$height != 'auto'">height: <xsl:value-of select="$width"/>px;</xsl:if>
										</xsl:attribute>
										<xsl:choose>
											<xsl:when test="@type = 'flash'">
												<div style="background: white;"><div id="{$id}" data-dbid="{@DB_ID}" data-width="{$width}" data-height="{$height}" data-media-type="{@type}" data-type="{$type}">
													<xsl:attribute name="class">flash-media-object-placeholder<xsl:if test="@fs = 'yes'"> eAu-object-full-screen</xsl:if></xsl:attribute>
												</div></div>
											</xsl:when>
											<xsl:when test="@type = 'video'">
												<div style="background: white;"><div id="{$id}" data-dbid="{@DB_ID}" data-width="{$width}" data-height="{$height}" data-media-type="{@type}" data-type="{$type}">
													<xsl:attribute name="class">flash-media-object-placeholder<xsl:if test="@fs = 'yes'"> eAu-object-full-screen</xsl:if></xsl:attribute>
												</div></div>
											</xsl:when>
											<xsl:when test="@type = 'sound'">
												<div style="background: white;"><div id="{$id}" data-dbid="{@DB_ID}" data-width="{$width}" data-height="{$height}" data-media-type="{@type}" data-type="{$type}" class="flash-media-object-placeholder">
													<xsl:attribute name="class">flash-media-object-placeholder<xsl:if test="@fs = 'yes'"> eAu-object-full-screen</xsl:if></xsl:attribute>
												</div></div>
											</xsl:when>
											<xsl:otherwise>
												<div id="{$id}" data-dbid="{@DB_ID}" data-width="{$width}" data-height="{$height}" data-media-type="{@type}" data-type="{$type}">
													<xsl:attribute name="class">media-object-placeholder<xsl:if test="@fs = 'yes'"> eAu-object-full-screen</xsl:if></xsl:attribute>
													<iframe src="javascript:void(0);" id="iframe-{$id}"/>
												</div>
											</xsl:otherwise>
										</xsl:choose>
									</div>
								</div>
							</xsl:otherwise>
						</xsl:choose>
					</div>
				</xsl:if>
			</div>
		</xsl:if>
	</xsl:template>

	<!-- Универсальный шаблон для отображения текстовой ноды -->
	<xsl:template match="text">
		<xsl:param name="hide-nodes"/>
		<xsl:if test="not(@visibility = 'hidden')">
			<div class="eAu-paragraph-container">
				<xsl:attribute name="id">
					<xsl:call-template name="genID">
						<xsl:with-param name="type">eAu-node</xsl:with-param>
						<xsl:with-param name="id"><xsl:value-of select="@DB_ID"/></xsl:with-param>
					</xsl:call-template>
				</xsl:attribute>
				<xsl:if test="$hide-nodes = 'yes'">
					<xsl:attribute name="style">display: none;</xsl:attribute>
				</xsl:if>
				<xsl:if test="($element-title and not(../@show-title = 'no'))">
					<h1 class="eAu">
						<xsl:if test="$element-title and not(../@show-title = 'no')"><xsl:value-of select="$element-title"/></xsl:if>
						<xsl:if test="$parent-id"><a href="index.htm" db-id="{$parent-id}" data-eau-target="navigate" onclick="return false;" target="_top" title="Вернуться к основному содержимому"><img border="0" src="images/return.gif" data-eau-target="navigate" width="18" height="18" align="absmiddle" hspace="7" /></a></xsl:if>
					</h1>
				</xsl:if>
				<xsl:if test="@title and normalize-space(@title) != '' and @show-title = 'yes'">
					<h2 class="eAu"><xsl:value-of select="@title"/></h2>
				</xsl:if>
				<div class="eAu-paragraph">
					<xsl:value-of select="."/>
				</div>
			</div>
		</xsl:if>
	</xsl:template>

	<!-- Корневой элемент -->
	<xsl:template match="organization" priority="100">
		<xsl:choose>
			<xsl:when test="$transformed-element-id and not($transformed-element-id = @DB_ID)">
				<div><xsl:apply-templates select="//*[@DB_ID = $transformed-element-id]"/></div>
			</xsl:when>
			<xsl:otherwise>
				<div class="eAu-organization-container">
					<div class="eAu-organization">
						<xsl:choose>
							<xsl:when test="intro"><xsl:apply-templates select="intro[position() = 1 and not(@visibility = 'hidden')]" /></xsl:when>
							<xsl:otherwise>
								<div class="eAu-intro-container">
									<div class="eAu-intro">
										Для начала работы выберите из содержания текущего курса соответствующий раздел
									</div>
								</div>
							</xsl:otherwise>
						</xsl:choose>
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Вводный экран -->
	<xsl:template match="intro">
		<div class="eAu-intro-container">
			<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><h1 class="eAu"><xsl:value-of select="@title" /></h1></xsl:if>
			<div class="eAu-intro">
				<xsl:apply-templates select="text[not(@visibility = 'hidden')] | object[not(@visibility = 'hidden')]"/>
			</div>
		</div>
	</xsl:template>

	<!-- отображает раздел -->
	<xsl:template match="unit">
		<div class="eAu-item-container eAu-unit-container">
			<xsl:choose>
				<xsl:when test="$transformed-element-id = @DB_ID">
					<xsl:if test="(@title and normalize-space(@title) != '' and not(@show-title = 'no')) or $parent-id">
						<h1 class="eAu">
							<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><xsl:value-of select="@title"/></xsl:if>
							<xsl:if test="$parent-id"><a href="index.htm" db-id="{$parent-id}" onclick="return false;" target="_top" title="Вернуться к основному содержимому"><img border="0" src="images/return.gif" data-eau-target="navigate" width="18" height="18" align="absmiddle" hspace="7" /></a></xsl:if>
						</h1>
					</xsl:if>
					<div class="eAu-item eAu-unit">
						<xsl:apply-templates select="references[not(@visibility = 'hidden')] | subject[not(@visibility = 'hidden')] | unit[not(@visibility = 'hidden')] | lesson[not(@visibility = 'hidden')] | test[not(@visibility = 'hidden')] | studiedproblems[not(@visibility = 'hidden')] | studiedproblem[not(@visibility = 'hidden')]">
							<xsl:with-param name="mode">unit</xsl:with-param>
						</xsl:apply-templates>
					</div>
				</xsl:when>
				<xsl:otherwise>
					<div class="eAu"><a href="index.htm" db-id="{@DB_ID}" onclick="return false;" target="_top"><xsl:value-of select="@title" /></a></div>
				</xsl:otherwise>
			</xsl:choose>
		</div>
	</xsl:template>

	<!-- отображает модуль (вид зависит от режима) -->
	<xsl:template match="lesson">
		<xsl:param name="mode" />
		<xsl:choose>
			<xsl:when test="$mode = 'unit'">
				<div class="eAu-item-container eAu-lesson-container">
					<div class="eAu">
						<!-- <a href="index.htm" onclick="toggleDescription('EAU-EXPANDABLE-{@DB_ID}', this); return false;" target="_top" title="Развернуть описание" class="eAu-expand-collapse"><span>[РАЗВЕРНУТЬ/СВЕРНУТЬ]</span></a> -->
						<a href="index.htm" db-id="{@DB_ID}" onclick="return false;" target="_top"><xsl:value-of select="@title" /></a>
					</div>
					<!-- <div class="eAu-item eAu-lesson" style="display: none;" id="EAU-EXPANDABLE-{@DB_ID}">
						<xsl:apply-templates select="subject[not(@visibility = 'hidden')] | references[not(@visibility = 'hidden')]" />
						<xsl:apply-templates select="test[not(@visibility = 'hidden')] | studiedproblems[not(@visibility = 'hidden')] | studiedproblem[not(@visibility = 'hidden')]">
							<xsl:with-param name="mode">unit</xsl:with-param>
						</xsl:apply-templates>
					</div> -->
				</div>
			</xsl:when>
			<xsl:when test="$mode = 'appendix' or ancestor::appendix-item">
				<div class="eAu-appendix-lesson">
					<xsl:call-template name="modules">
						<xsl:with-param name="node" select="." />
					</xsl:call-template>
				</div>
			</xsl:when>
			<xsl:otherwise>
				<div class="eAu-lesson-container">
					<xsl:if test="(@title and normalize-space(@title) != '' and not(@show-title = 'no')) or $parent-id">
						<h1 class="eAu">
							<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><xsl:value-of select="@title"/></xsl:if>
							<xsl:if test="$parent-id"><a href="index.htm" db-id="{$parent-id}" onclick="return false;" target="_top" title="Вернуться к основному содержимому"><img border="0" src="images/return.gif" data-eau-target="navigate" width="18" height="18" align="absmiddle" hspace="7" /></a></xsl:if>
						</h1>
					</xsl:if>
					<div class="eAu-lesson">
						<xsl:apply-templates select="subject[not(@visibility = 'hidden')] | references[not(@visibility = 'hidden')]" />
						<xsl:apply-templates select="test[not(@visibility = 'hidden')] | studiedproblems[not(@visibility = 'hidden')] | studiedproblem[not(@visibility = 'hidden')]">
							<xsl:with-param name="mode">lesson</xsl:with-param>
						</xsl:apply-templates>
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="appendixes">
		<div class="eAu-appendixes-container">
			<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><h1 class="eAu"><xsl:value-of select="@title" /></h1></xsl:if>
			<div class="eAu-appendixes">
				<xsl:if test="glossary"><h1 class="eAu">Глоссарий</h1></xsl:if>
				<xsl:apply-templates select="appendix-item" />
			</div>
		</div>
	</xsl:template>

	<!-- Тема модуля/раздела -->
	<xsl:template match="subject">
		<div class="eAu-subject-container">
			<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><h1 class="eAu"><xsl:value-of select="@title" /></h1></xsl:if>
			<div class="eAu-subject"><xsl:value-of select="." /></div>
		</div>
	</xsl:template>

	<!-- Список литературы -->
	<xsl:template match="references">
		<div class="eAu-references-container">
			<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><h1 class="eAu"><xsl:value-of select="@title" /></h1></xsl:if>
			<ol class="eAu-references">
				<xsl:for-each select="reference[not(@visibility = 'hidden')] | object[not(@visibility = 'hidden')]">
					<xsl:choose>
						<xsl:when test="local-name() = 'reference'">
							<xsl:apply-templates select="."/>
						</xsl:when>
						<xsl:otherwise>
							<li class="eAu-make-inline eAu-reference"><xsl:apply-templates select="."/></li>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</ol>
		</div>
	</xsl:template>
	<xsl:template match="reference">
		<li class="eAu-make-inline eAu-reference"><xsl:value-of select="." /></li>
	</xsl:template>

	<!-- Ссылка на тест в unit-view и lesson-view -->
	<xsl:template match="test">
		<xsl:param name="mode" />
		<xsl:choose>
			<xsl:when test="$mode = 'unit' or $mode = 'lesson'">
				<div class="eAu-test-link"><a href="index.htm" db-id="{@DB_ID}" onclick="return false;" target="_top"><xsl:value-of select="@title" /></a></div>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<!-- Список тем -->
	<xsl:template match="studiedproblems">
		<xsl:param name="mode" />
		<xsl:choose>
			<xsl:when test="$mode = 'unit' or $mode = 'lesson'">
				<xsl:if test="count(studiedproblem[not(@visibility = 'hidden')]) > 0">
					<div class="eAu-studiedproblems-container">
						<xsl:if test="@title and normalize-space(@title) != ''">
							<div class="eAu-studiedproblems-container-legend eAu"><xsl:value-of select="@title" /></div>
						</xsl:if>
						<div class="eAu-studiedproblems">
							<xsl:apply-templates select="studiedproblem[not(@visibility = 'hidden')]">
								<xsl:with-param name="mode" select="$mode" />
							</xsl:apply-templates>
						</div>
					</div>
				</xsl:if>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<!-- Отображает тему в режиме обучения -->
	<!-- если показывается в unit-view или lesson-view, -->
	<!-- то отображается только ссылка -->
	<xsl:template match="studiedproblem">
		<xsl:param name="mode" />
		<xsl:choose>
			<xsl:when test="$mode = 'unit' or $mode = 'lesson'">
				<div class="eAu-studiedproblem-link"><a href="index.htm" db-id="{@DB_ID}" onclick="return false;" target="_top" class="some-cool-clasZ"><xsl:value-of select="@title"/></a></div>
			</xsl:when>
			<xsl:when test="$mode = 'appendix' or ancestor::appendix-item">
				<xsl:call-template name="studiedproblem"><xsl:with-param name="node" select="."/></xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<div class="eAu-studiedproblem-container">
					<xsl:if test="(@title and normalize-space(@title) != '' and not(@show-title = 'no')) or $parent-id">
						<h1 class="eAu">
							<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><xsl:value-of select="@title" /></xsl:if>
							<xsl:if test="$parent-id"><a href="index.htm" db-id="{$parent-id}" onclick="return false;" target="_top" title="Вернуться к основному содержимому"><img border="0" src="images/return.gif" data-eau-target="navigate" width="18" height="18" align="absmiddle" hspace="7" /></a></xsl:if>
						</h1>
					</xsl:if>
					<div class="eAu-studiedproblem">
						<xsl:choose>
							<xsl:when test="@delay and normalize-space(@delay) != '' and number(normalize-space(@delay)) &gt; 0">
								<xsl:apply-templates select="text[not(@visibility = 'hidden')] | object[not(@visibility = 'hidden')]">
										<xsl:with-param name="hide-nodes">yes</xsl:with-param>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<xsl:apply-templates select="text[not(@visibility = 'hidden')] | object[not(@visibility = 'hidden')]"/>
							</xsl:otherwise>
						</xsl:choose>
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Отображает тему в приложении -->
	<xsl:template name="studiedproblem">
		<xsl:param name="heading" select="'h1'"/>
		<xsl:param name="node"/>
		<div class="eAu-studiedproblem-container">
			<xsl:element name="{$heading}">
				<xsl:attribute name="class">eAu</xsl:attribute>
				<xsl:value-of select="$node/@title" />
			</xsl:element>
			<div class="eAu-studiedproblem"><xsl:apply-templates select="$node/*[not(@visibility = 'hidden')]" /></div>
		</div>
	</xsl:template>

	<!-- Отображает список тем внутри модуля -->
	<xsl:template name="modules">
		<xsl:param name="node" />
		<div class="eAu-modules-container">
			<xsl:if test="$node/@title and normalize-space($node/@title) != '' and not($node/@show-title = 'no')">
				<h1 class="eAu"><xsl:value-of select="$node/@title" /></h1>
			</xsl:if>
			<div class="eAu-modules">
				<xsl:for-each select="$node/*">
					<xsl:if test="name(.) = 'lesson'">
						<div class="eAu-appendix-lesson">
							<xsl:call-template name="modules">
								<xsl:with-param name="node" select="." />
							</xsl:call-template>
						</div>
					</xsl:if>
					<xsl:if test="name(.) = 'studiedproblem'">
						<xsl:call-template name="studiedproblem">
							<xsl:with-param name="node" select="."/>
							<xsl:with-param name="heading" select="'h2'"/>
						</xsl:call-template>
						<xsl:if test="position() != count(../*)">
							<div class="eAu-content-dotted-line"></div>
						</xsl:if>
					</xsl:if>
				</xsl:for-each>
			</div>
		</div>
	</xsl:template>

	<!-- Корневой элемент приложения -->
	<xsl:template match="appendix-item">
		<xsl:choose>
			<xsl:when test="intro"><xsl:apply-templates select="intro[position() = 1 and not(@visibility = 'hidden')]" /></xsl:when>
			<xsl:otherwise>
				<div class="eAu-intro-container">
					<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')">
						<h1 class="eAu"><xsl:value-of select="@title"/></h1>					
					</xsl:if>
					<div class="eAu-intro">
						Для продолжения работы выберите из содержания текущего приложения соответствующий раздел
					</div>
				</div>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="test-statistics">
		<!-- TODO общая статистика за тест -->
		<h1 class="eAu" id="eAu-test-title"><xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')">
			<xsl:value-of select="@title"/>
		</xsl:if></h1>
		<div id="eAu-test-statistics" class="eAu-test-statistics">
			<table cellpadding="0" cellspacing="0" style="border-collapse: collapse;" border="0" width="100%">
				<tr><td>
					<div id="eAu-test-questions-statistics" class="eAu-test-questions-statistics"></div>
					<div id="eAu-test-time" class="eAu-test-time"></div>
				</td>
				<td align="right">
					<div style="display: none;" id="eAu-test-skip" class="eAu-test-skip">
						<a href="index.htm" target="_top">прервать тестирование</a>
					</div>
					<div style="display: none;" id="eAu-test-interrupt" class="eAu-test-interrupt">
						<a href="index.htm" target="_top">досрочно завершить тестирование</a>
					</div>
				</td></tr>
			</table>
		</div>
	</xsl:template>
	
	<xsl:template match="test[@eAu-transform-group = 'yes']">
		<div class="eAu-test-container">
			<xsl:call-template name="test-statistics" />
			
			<xsl:apply-templates />
		</div>
	</xsl:template>

	<!-- Вопросы для контроля -->
	<xsl:template match="*[@eAu-is-question = 'yes']">
		<div class="eAu-question-container eAu-question-type-{@type}" id="eAu-question-{@DB_ID}">
			<xsl:if test="@title and normalize-space(@title) != '' and not(@show-title = 'no')"><h2 class="eAu" id="eAu-question-title-{@DB_ID}"><xsl:value-of select="@title"/></h2></xsl:if>
			<div id="eAu-question-user-answer-summary-{@DB_ID}" style="display: none;" class="eAu-question-user-answer-summary"></div>
			<div id="eAu-question-statistics-{@DB_ID}" class="eAu-question-statistics" style="display: none;">
				<xsl:if test="@group-name != ''">
					<div id="eAu-question-theme-{@DB_ID}" class="eAu-question-theme">тема вопроса <![CDATA[&ndash;]]> <span><xsl:value-of select="@group-name" /></span></div>
				</xsl:if>
				<div id="eAu-question-time-{@DB_ID}" class="eAu-quesion-time"></div>
			</div>
			<xsl:if test="text and normalize-space(text) != '' and normalize-space(text) != ' '">
				<div id="eAu-question-text-{@DB_ID}" class="eAu-question-text"><xsl:value-of select="text"/></div>
			</xsl:if>
			<div class="eAu-question-answers eAu-question-answers-interactive" id="eAu-question-answers-{@DB_ID}">
				<xsl:apply-templates select="answers | object">
					<xsl:with-param name="type">test</xsl:with-param>
				</xsl:apply-templates>
			</div>
		</div>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes']/answers">
		<form action="index.htm" method="POST" id="eAu-question-form-{../@DB_ID}" name="eAu-question-form" onsubmit="return false;" class="eAu-question-form">
			<xsl:choose>
				<xsl:when test="../@type = 'single' or ../@type = 'multiple'">
					<ul class="eAu-question-answers-simple">
						<xsl:apply-templates select="answer"/>
					</ul>
				</xsl:when>
				<xsl:otherwise>
					<table class="eAu-question-answers-table">
						<xsl:apply-templates select="answer"/>
					</table>
				</xsl:otherwise>
			</xsl:choose>
			
		</form>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and (@type='single' or @type='multiple')]/answers/answer">
		<xsl:variable name="DB_ID"><xsl:value-of select="../../@DB_ID"/></xsl:variable>
		<li class="eAu-question-answer-item eAu-answer-row">
			<div class="eAu-answer-input-cell eAu-answer-cell">
				<input name="EU-{$DB_ID}" id="EU-{$DB_ID}-{@DB_ID}" value="{@DB_ID}" class="eAu-input-radio eAu-input-checkbox">
					<xsl:attribute name="type">
						<xsl:choose>
							<xsl:when test="../../@type = 'single'"><xsl:text>radio</xsl:text></xsl:when>
							<xsl:otherwise><xsl:text>checkbox</xsl:text></xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				</input>
			</div>
			<div class="eAu-answer-variant eAu-answer-variant-cell eAu-answer-cell">
				<div>
					<xsl:choose>
						<xsl:when test="normalize-space(.) = ''">
							<i style="text-decoration: line-through;">тект ответа отсутствует</i>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="."/>
						</xsl:otherwise>
					</xsl:choose>
				</div>
			</div>
			<div class="eAu-answer-indicator eAu-answer-indicator-cell eAu-answer-cell"></div>
		</li>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and @type='fill']/answers/answer">
		<tr class="eAu-answer-row">
			<td class="eAu-answer-variant"><xsl:if test="not(normalize-space(.) = '')"><div><xsl:value-of select="."/></div></xsl:if></td>
			<td><xsl:if test="not(normalize-space(.) = '')"><span class="eAu-arrow">→</span></xsl:if></td>
			<td class="eAu-answer-input-cell"><input type="text" class="eAu-input-text" width="40" is-gap="yes" data-dbid="{@DB_ID}" /></td>
			<td class="eAu-answer-indicator"></td>
			<td id="EU-RA-{@DB_ID}"></td>
		</tr>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and @type='compare']/answers/answer">
		<tr class="eAu-answer-row">
			<td class="eAu-answer-variant">
				<xsl:choose>
					<xsl:when test="normalize-space(.) = ''">
						<i style="text-decoration: line-through;">тект ответа отсутствует</i>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="."/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td><span class="eAu-arrow">→</span></td>
			<td class="eAu-answer-input-cell">
				<select is-gap="yes" data-dbid="{@DB_ID}" class="eAu-input-select"></select>
			</td>
			<td class="eAu-answer-indicator"></td>
			<td id="EU-RA-{@DB_ID}"></td>
		</tr>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and @type='fillingaps']/answers" priority="10">
		<div id="eAu-question-form-{../@DB_ID}" class="eAu-question-form">
			<!-- Может-быть здесь можно разместить незадействованные элементы -->
		</div>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and @type='sort']/answers" priority="10">
		<xsl:variable name="sortable_id"><xsl:call-template name="genID">
			<xsl:with-param name="id" select="../@DB_ID"/>
		</xsl:call-template></xsl:variable>
		<xsl:variable name="DB_ID"><xsl:value-of select="../@DB_ID"/></xsl:variable>
		
		<div id="eAu-question-sortables-area-{$DB_ID}" class="eAu-question-sortables-area eAu-question-sort eAu-question-answers-draggable">
			<div class="eAu-sortable-container">
				<div id="EU-SORT-{$DB_ID}" class="eAu-sortable eAu-sortable-{$sortable_id}">
					<xsl:for-each select="answer">
						<div class="eAu-sortable-item" id="{@DB_ID}">
							<div class="eAu-sortable-item-container">
								<table cellpadding="0" cellspacing="0" border="0" width="100%"><tr>
									<td class="eAu-answer-variant">
										<xsl:choose>
											<xsl:when test="normalize-space(.) = ''">
												<i style="text-decoration: line-through;">тект ответа отсутствует</i>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="."/>
											</xsl:otherwise>
										</xsl:choose>
									</td>
									<td class="eAu-answer-indicator"></td>
									<td class="eAu-answer-right" id="EU-RA-{@DB_ID}"></td>
								</tr></table>
							</div>
						</div>
					</xsl:for-each>
				</div>
			</div>
		</div>
	</xsl:template>

	<xsl:template name="classify-group-title">
		<xsl:param name="value"/>
		<h3 class="eAu">
			<xsl:choose>
				<xsl:when test="$value = ''">
					<i>Категория без имени</i>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$value"/>
				</xsl:otherwise>
			</xsl:choose>
		</h3>
	</xsl:template>

	<xsl:template match="*[@eAu-is-question = 'yes' and @type='classify']/answers" priority="10">
		<xsl:variable name="sortable_id"><xsl:call-template name="genID">
			<xsl:with-param name="id" select="../@DB_ID"/>
		</xsl:call-template></xsl:variable>
		<xsl:variable name="DB_ID"><xsl:value-of select="../@DB_ID"/></xsl:variable>
		
		<div id="eAu-question-sortables-area-{$DB_ID}" class="eAu-question-sortables-area eAu-question-classify eAu-question-answers-draggable">
			<xsl:for-each select="../class-group">
				<xsl:call-template name="classify-group-title">
					<xsl:with-param name="value" select="@class-group"/>
				</xsl:call-template>
				<div class="eAu-sortable-container">
					<div id="EU-CLASSIFY-{$DB_ID}-{@class-group-id}" class="eAu-sortable eAu-sortable-useranswer eAu-sortable-{$sortable_id}" group="{@class-group}">
					</div>
				</div>
			</xsl:for-each>
			<div class="eAu-sortable-container eAu-sortable-classify-box">
				<div id="EU-CLASSIFY-{$DB_ID}" class="eAu-sortable eAu-sortable-{$sortable_id}">
					<xsl:for-each select="answer">
						<div class="eAu-sortable-item" id="{@DB_ID}">
							<div class="eAu-sortable-item-container">
								<table cellpadding="0" cellspacing="0" border="0" width="100%"><tr>
									<td class="eAu-answer-variant">
										<xsl:choose>
											<xsl:when test="normalize-space(.) = ''">
												<i style="text-decoration: line-through;">тект ответа отсутствует</i>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="."/>
											</xsl:otherwise>
										</xsl:choose>
									</td>
									<td width="1" class="eAu-answer-right" id="EU-RA-{@DB_ID}"></td>
									<td class="eAu-answer-indicator"></td>
								</tr></table>
							</div>
						</div>
					</xsl:for-each>
				</div>
			</div>
		</div>
	</xsl:template>
</xsl:stylesheet>
